@@ -5,14 +5,18 @@ from OgreInterface.surfaces import Surface, Interface
 from OgreInterface import utils
 from OgreInterface.lattice_match import ZurMcGill, OgreMatch
 
-from pymatgen.core.structure import Structure
-from pymatgen.analysis.structure_matcher import StructureMatcher
+from pymatgen.core.structure import Structure, Molecule
+from pymatgen.core.periodic_table import Element, DummySpecies
 from pymatgen.core.lattice import Lattice
 from pymatgen.io.vasp.inputs import Poscar
 from pymatgen.io.ase import AseAtomsAdaptor
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
-from pymatgen.analysis.interfaces.zsl import ZSLGenerator
 from pymatgen.core.operations import SymmOp
+from pymatgen.analysis.graphs import StructureGraph
+from pymatgen.analysis.local_env import JmolNN
+import networkx as nx
+import numpy as np
+
 
 from tqdm import tqdm
 import numpy as np
@@ -21,8 +25,10 @@ from copy import deepcopy
 from typing import Union, List, TypeVar
 from itertools import combinations, product, groupby
 from ase import Atoms
+from ase.data import chemical_symbols
 from multiprocessing import Pool, cpu_count
 import time
+from collections.abc import Sequence
 
 from scipy.cluster.hierarchy import fcluster, linkage
 from scipy.spatial.distance import squareform
@@ -30,6 +36,11 @@ from scipy.spatial.distance import squareform
 SelfSurfaceGenerator = TypeVar(
     "SelfSurfaceGenerator", bound="SurfaceGenerator"
 )
+
+SelfOrganicSurfaceGenerator = TypeVar(
+    "SelfOrganicSurfaceGenerator", bound="OrganicSurfaceGenerator"
+)
+
 SelfInterfaceGenerator = TypeVar(
     "SelfInterfaceGenerator", bound="InterfaceGenerator"
 )
@@ -41,7 +52,7 @@ class TolarenceError(RuntimeError):
     pass
 
 
-class SurfaceGenerator:
+class SurfaceGenerator(Sequence):
     """Class for generating surfaces from a given bulk structure.
 
     The SurfaceGenerator classes generates surfaces with all possible terminations and contains
@@ -108,6 +119,7 @@ class SurfaceGenerator:
         filter_ionic_slabs: bool = False,
         lazy: bool = False,
     ) -> None:
+        super().__init__()
         self.convert_to_conventional = convert_to_conventional
 
         (
@@ -140,12 +152,25 @@ class SurfaceGenerator:
         ) = self._get_oriented_bulk_structure()
 
         if not self.lazy:
-            self.slabs = self._generate_slabs()
+            self._slabs = self._generate_slabs()
+        else:
+            self._slabs = None
+
+    def __getitem__(self, i):
+        if self._slabs:
+            return self._slabs[i]
+        else:
+            print(
+                "The slabs have not been generated yet, please use the generate_slabs() function to create them."
+            )
+
+    def __len__(self):
+        return len(self._slabs)
 
     def generate_slabs(self) -> None:
         """Used to generate list of Surface objects if lazy=True"""
         if self.lazy:
-            self.slabs = self._generate_slabs()
+            self._slabs = self._generate_slabs()
         else:
             print(
                 "The slabs are already generated upon initialization. This function is only needed if lazy=True"
@@ -202,12 +227,14 @@ class SurfaceGenerator:
                 f"structure accepts 'pymatgen.core.structure.Structure' or 'ase.Atoms' not '{type(atoms_or_struc).__name__}'"
             )
 
-        sg = SpacegroupAnalyzer(init_structure)
-        prim_structure = sg.get_primitive_standard_structure()
-        prim_atoms = AseAtomsAdaptor.get_atoms(prim_structure)
-
         if self.convert_to_conventional:
+            sg = SpacegroupAnalyzer(init_structure)
             conventional_structure = sg.get_conventional_standard_structure()
+            prim_structure = self._add_symmetry_info(
+                conventional_structure, return_primitive=True
+            )
+            # prim_structure = sg.get_primitive_standard_structure()
+            prim_atoms = AseAtomsAdaptor.get_atoms(prim_structure)
             conventional_atoms = AseAtomsAdaptor.get_atoms(
                 conventional_structure
             )
@@ -219,8 +246,20 @@ class SurfaceGenerator:
                 prim_atoms,
             )
         else:
+            if "molecule_index" in init_structure.site_properties:
+                self._add_symmetry_info_molecule(init_structure)
+            else:
+                self._add_symmetry_info(init_structure, return_primitive=False)
+
             init_atoms = AseAtomsAdaptor().get_atoms(init_structure)
 
+            prim_structure = init_structure.get_primitive_structure()
+            prim_atoms = AseAtomsAdaptor().get_atoms(prim_structure)
+
+            # utils._get_colored_molecules(
+            #     prim_structure, "vis/POSCAR_mol_prim_color"
+            # )
+
             return init_structure, init_atoms, prim_structure, prim_atoms
 
     def _get_point_group_operations(self):
@@ -233,6 +272,41 @@ class SurfaceGenerator:
 
         return unique_operations
 
+    def _add_symmetry_info(self, struc, return_primitive=False):
+        sg = SpacegroupAnalyzer(struc)
+        dataset = sg.get_symmetry_dataset()
+        struc.add_site_property("bulk_wyckoff", dataset["wyckoffs"])
+        struc.add_site_property(
+            "bulk_equivalent",
+            dataset["equivalent_atoms"].tolist(),
+        )
+
+        if return_primitive:
+            prim_mapping = dataset["mapping_to_primitive"]
+            _, prim_inds = np.unique(prim_mapping, return_index=True)
+            prim_bulk = sg.get_primitive_standard_structure()
+
+            prim_bulk.add_site_property(
+                "bulk_wyckoff",
+                [dataset["wyckoffs"][i] for i in prim_inds],
+            )
+            prim_bulk.add_site_property(
+                "bulk_equivalent",
+                dataset["equivalent_atoms"][prim_inds].tolist(),
+            )
+
+            return prim_bulk
+
+    def _add_symmetry_info_molecule(self, struc):
+        sg = SpacegroupAnalyzer(struc)
+        dataset = sg.get_symmetry_dataset()
+        wyckoffs = dataset["wyckoffs"]
+        equivalent_atoms = dataset["equivalent_atoms"]
+        molecule_index = struc.site_properties["molecule_index"]
+        equivalent_molecules = [molecule_index[i] for i in equivalent_atoms]
+        struc.add_site_property("bulk_wyckoff", wyckoffs)
+        struc.add_site_property("bulk_equivalent", equivalent_molecules)
+
     def _get_oriented_bulk_structure(self):
         bulk = self.bulk_structure
         prim_bulk = self.primitive_structure
@@ -241,41 +315,24 @@ class SurfaceGenerator:
         prim_lattice = prim_bulk.lattice
 
         recip_lattice = lattice.reciprocal_lattice_crystallographic
-        prim_recip_lattice = prim_lattice.reciprocal_lattice_crystallographic
 
         miller_index = self.miller_index
 
         d_hkl = lattice.d_hkl(miller_index)
-        normal_vector = recip_lattice.get_cartesian_coords(miller_index)
-        prim_miller_index = prim_recip_lattice.get_fractional_coords(
-            normal_vector
+
+        normal_vector = lattice.get_cartesian_coords(
+            np.array(miller_index).dot(recip_lattice.metric_tensor)
         )
+        prim_normal_vector = prim_lattice.get_fractional_coords(normal_vector)
+        prim_miller_index = prim_normal_vector.dot(prim_lattice.metric_tensor)
+        prim_miller_index = prim_miller_index
+
         prim_miller_index = utils._get_reduced_vector(
             prim_miller_index
         ).astype(int)
 
         normal_vector /= np.linalg.norm(normal_vector)
 
-        sg = SpacegroupAnalyzer(bulk)
-        prim_mapping = sg.get_symmetry_dataset()["mapping_to_primitive"]
-        _, prim_inds = np.unique(prim_mapping, return_index=True)
-
-        bulk.add_site_property(
-            "bulk_wyckoff", sg.get_symmetry_dataset()["wyckoffs"]
-        )
-        bulk.add_site_property(
-            "bulk_equivalent",
-            sg.get_symmetry_dataset()["equivalent_atoms"].tolist(),
-        )
-        prim_bulk.add_site_property(
-            "bulk_wyckoff",
-            [sg.get_symmetry_dataset()["wyckoffs"][i] for i in prim_inds],
-        )
-        prim_bulk.add_site_property(
-            "bulk_equivalent",
-            sg.get_symmetry_dataset()["equivalent_atoms"][prim_inds].tolist(),
-        )
-
         if not self._use_prim:
             intercepts = np.array(
                 [1 / i if i != 0 else 0 for i in miller_index]
@@ -294,6 +351,9 @@ class SurfaceGenerator:
 
         if len(non_zero_points) == 1:
             basis = np.eye(3)
+            basis[non_zero_points[0], non_zero_points[0]] *= intercepts[
+                non_zero_points[0]
+            ]
             dot_products = basis.dot(normal_vector)
             sort_inds = np.argsort(dot_products)
             basis = basis[sort_inds]
@@ -403,6 +463,15 @@ class SurfaceGenerator:
             ortho_basis, translation_vec=np.zeros(3)
         )
 
+        # if "molecules" in init_oriented_struc.site_properties:
+        #     for site in init_oriented_struc:
+        #         mol = site.properties["molecules"]
+        #         planar_mol = mol.copy()
+        #         planar_mol.translate_sites(range(len(mol)), site.coords)
+        #         planar_mol.apply_operation(to_planar_operation)
+        #         centered_mol = planar_mol.get_centered_molecule()
+        #         site.properties["molecules"] = centered_mol
+
         planar_oriented_struc = init_oriented_struc.copy()
         planar_oriented_struc.apply_operation(to_planar_operation)
 
@@ -425,12 +494,23 @@ class SurfaceGenerator:
         a_to_i_operation = SymmOp.from_rotation_and_translation(
             a_to_i.T, translation_vec=np.zeros(3)
         )
+
+        # if "molecules" in planar_oriented_struc.site_properties:
+        #     for site in planar_oriented_struc:
+        #         mol = site.properties["molecules"]
+        #         a_to_i_mol = mol.copy()
+        #         a_to_i_mol.translate_sites(range(len(mol)), site.coords)
+        #         a_to_i_mol.apply_operation(a_to_i_operation)
+        #         centered_mol = a_to_i_mol.get_centered_molecule()
+        #         site.properties["molecules"] = centered_mol
+
         planar_oriented_struc.apply_operation(a_to_i_operation)
 
-        planar_oriented_struc.add_site_property(
-            "oriented_bulk_equivalent",
-            list(range(len(planar_oriented_struc))),
-        )
+        if "molecule_index" not in planar_oriented_struc.site_properties:
+            planar_oriented_struc.add_site_property(
+                "oriented_bulk_equivalent",
+                list(range(len(planar_oriented_struc))),
+            )
 
         planar_oriented_struc.sort()
 
@@ -603,20 +683,30 @@ class SurfaceGenerator:
         )
         orthogonal_slab.sort()
 
-        shift = 0.5 * (vacuum_scale / (vacuum_scale + self.layers))
+        center_shift = 0.5 * (vacuum_scale / (vacuum_scale + self.layers))
         non_orthogonal_slab.translate_sites(
             indices=range(len(non_orthogonal_slab)),
-            vector=[0, 0, shift],
+            vector=[0, 0, center_shift],
             frac_coords=True,
             to_unit_cell=True,
         )
         orthogonal_slab.translate_sites(
             indices=range(len(orthogonal_slab)),
-            vector=[0, 0, shift],
+            vector=[0, 0, center_shift],
             frac_coords=True,
             to_unit_cell=True,
         )
 
+        # shift_str = f"{shift:.3f}".replace(".", "")
+        # Poscar(non_orthogonal_slab).write_file(f"POSCAR_slab_com_{shift_str}")
+
+        # if "molecules" in slab_base.site_properties:
+        #     slab_base = utils.add_molecules(slab_base)
+        #     orthogonal_slab = utils.add_molecules(orthogonal_slab)
+        #     non_orthogonal_slab = utils.add_molecules(non_orthogonal_slab)
+
+        # Poscar(non_orthogonal_slab).write_file(f"POSCAR_slab_mol_{shift_str}")
+
         return (
             slab_base,
             orthogonal_slab,
@@ -626,15 +716,6 @@ class SurfaceGenerator:
             vacuum,
         )
 
-    # def _get_ewald_energy(self, slab):
-    #     slab = deepcopy(slab)
-    #     bulk = deepcopy(self.pmg_structure)
-    #     slab.add_oxidation_state_by_guess()
-    #     bulk.add_oxidation_state_by_guess()
-    #     E_slab = EwaldSummation(slab).total_energy
-    #     E_bulk = EwaldSummation(bulk).total_energy
-    #     return E_slab, E_bulk
-
     def _generate_slabs(self):
         """
         This function is used to generate slab structures with all unique
@@ -717,7 +798,7 @@ class SurfaceGenerator:
         return surfaces
 
     def __len__(self):
-        return len(self.slabs)
+        return len(self._slabs)
 
     @property
     def nslabs(self):
@@ -731,7 +812,184 @@ class SurfaceGenerator:
         """
         Return the terminations of each slab generated by the SurfaceGenerator
         """
-        return {i: slab.get_termination() for i, slab in enumerate(self.slabs)}
+        return {
+            i: slab.get_termination() for i, slab in enumerate(self._slabs)
+        }
+
+
+class OrganicSurfaceGenerator(SurfaceGenerator):
+    """Class for generating surfaces from a given bulk structure.
+
+    The SurfaceGenerator classes generates surfaces with all possible terminations and contains
+    information pertinent to generating interfaces with the InterfaceGenerator.
+
+    Examples:
+        Creating a SurfaceGenerator object using PyMatGen to load the structure:
+        >>> from OgreInterface.generate import SurfaceGenerator
+        >>> from pymatgen.core.structure import Structure
+        >>> bulk = Structure.from_file("POSCAR_bulk")
+        >>> surfaces = SurfaceGenerator(bulk=bulk, miller_index=[1, 1, 1], layers=5, vacuum=60)
+        >>> surface = surfaces.slabs[0] # OgreInterface.Surface object
+
+        Creating a SurfaceGenerator object using the build in from_file() method:
+        >>> from OgreInterface.generate import SurfaceGenerator
+        >>> surfaces = SurfaceGenerator.from_file(filename="POSCAR_bulk", miller_index=[1, 1, 1], layers=5, vacuum=60)
+        >>> surface = surfaces.slabs[0] # OgreInterface.Surface object
+
+    Args:
+        bulk: Bulk crystal structure used to create the surface
+        miller_index: Miller index of the surface
+        layers: Number of layers to include in the surface
+        vacuum: Size of the vacuum to include over the surface in Angstroms
+        generate_all: Determines if all possible surface terminations are generated.
+        lazy: Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
+            (this is used for the MillerIndex search to make things faster)
+
+    Attributes:
+        slabs (list): List of OgreInterface Surface objects with different surface terminations.
+        bulk_structure (Structure): Pymatgen Structure class for the conventional cell of the input bulk structure
+        bulk_atoms (Atoms): ASE Atoms class for the conventional cell of the input bulk structure
+        primitive_structure (Structure): Pymatgen Structure class for the primitive cell of the input bulk structure
+        primitive_atoms (Atoms): ASE Atoms class for the primitive cell of the input bulk structure
+        miller_index (list): Miller index of the surface
+        layers (int): Number of layers to include in the surface
+        vacuum (float): Size of the vacuum to include over the surface in Angstroms
+        generate_all (bool): Determines if all possible surface terminations are generated.
+        filter_ionic_slab (bool): Determines if the terminations of ionic crystals should be filtered out based on their
+            predicted stability calculated using the IonicScoreFunction
+        lazy (bool): Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
+            (this is used for the MillerIndex search to make things faster)
+        oriented_bulk_structure (Structure): Pymatgen Structure class of the smallest building block of the slab,
+            which will eventually be used to build the slab supercell
+        oriented_bulk_atoms (Atoms): Pymatgen Atoms class of the smallest building block of the slab,
+            which will eventually be used to build the slab supercell
+        uvw_basis (list): The miller indices of the slab lattice vectors.
+        transformation_matrix: Transformation matrix used to convert from the bulk basis to the slab basis
+            (usefull for band unfolding calculations)
+        inplane_vectors (list): The cartesian vectors of the in-plane lattice vectors.
+        surface_normal (list): The normal vector of the surface
+        c_projection (float): The projections of the c-lattice vector onto the surface normal
+    """
+
+    def __init__(
+        self,
+        bulk: Union[Structure, Atoms],
+        miller_index: List[int],
+        layers: int,
+        vacuum: float,
+        generate_all: bool = True,
+        lazy: bool = False,
+    ) -> None:
+        # Get the primitive structure to label the molecuels
+        prim_bulk = bulk.get_primitive_structure()
+
+        # Get the primitive to conventional transformation matrix
+        prim_to_conv = np.round(utils.conv_a_to_b(prim_bulk, bulk), 3)
+
+        if not np.isclose(
+            (np.round(prim_to_conv).astype(int) - prim_to_conv).sum(), 0.0
+        ):
+            print(
+                "WARNING: Something went wrong with reducing the structure to it's primitive cell"
+            )
+
+        # Label the molecules with dummy atoms
+        self._label_molecules(prim_bulk)
+
+        # Revert back to the supercell for the input into the SurfaceGenerator
+        labeled_bulk = prim_bulk.copy()
+        labeled_bulk.make_supercell(prim_to_conv)
+        # dummy_bulk = utils.replace_molecules_with_atoms(labeled_bulk)
+
+        # Poscar(labeled_bulk).write_file("vis/POSCAR_mol_bulk")
+        # utils._get_colored_molecules(
+        #     labeled_bulk, "vis/POSCAR_mol_bulk_colored"
+        # )
+        # utils._get_colored_molecules(dummy_bulk, "vis/POSCAR_com_bulk_colored")
+
+        super().__init__(
+            bulk=labeled_bulk,
+            miller_index=miller_index,
+            layers=layers,
+            vacuum=vacuum,
+            convert_to_conventional=False,
+            generate_all=generate_all,
+            filter_ionic_slabs=False,
+            lazy=True,
+        )
+
+        # Extract the oriented bulk structure
+        obs = self.oriented_bulk_structure
+        # utils._get_colored_molecules(obs, "vis/POSCAR_mol_obs_colored")
+
+        # Replace the molecules with their cooresponding dummy atoms at their center of mass
+        dummy_obs = utils.replace_molecules_with_atoms(obs)
+        # utils._get_colored_molecules(dummy_obs, "vis/POSCAR_com_obs_colored")
+
+        # Add oriented_bulk_equivalent site property
+        dummy_obs.add_site_property(
+            "oriented_bulk_equivalent", range(len(dummy_obs))
+        )
+
+        # Replace the oriented bulk structure from the SurfaceGenerator class with the dummy atoms
+        self.oriented_bulk_structure = dummy_obs
+
+        # Generate the surfaces
+        if not lazy:
+            self.generate_slabs()
+
+    @classmethod
+    def from_file(
+        cls,
+        filename: str,
+        miller_index: List[int],
+        layers: int,
+        vacuum: float,
+        generate_all: bool = True,
+        lazy: bool = False,
+    ) -> SelfOrganicSurfaceGenerator:
+        """Creating a OrganicSurfaceGenerator from a file (i.e. POSCAR, cif, etc)
+
+        Args:
+            filename: File path to the structure file
+            miller_index: Miller index of the surface
+            layers: Number of layers to include in the surface
+            vacuum: Size of the vacuum to include over the surface in Angstroms
+            generate_all: Determines if all possible surface terminations are generated
+            lazy: Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
+                (this is used for the MillerIndex search to make things faster)
+
+        Returns:
+            OrganicSurfaceGenerator
+        """
+        structure = Structure.from_file(filename=filename)
+
+        return cls(
+            structure,
+            miller_index,
+            layers,
+            vacuum,
+            generate_all,
+            lazy,
+        )
+
+    def _label_molecules(self, struc):
+        # Create a structure graph so we can extract the molecules
+        struc_graph = StructureGraph.with_local_env_strategy(struc, JmolNN())
+        graph = nx.Graph(struc_graph.graph)
+
+        # Extract all molecule subgraphs
+        subgraphs = [graph.subgraph(c) for c in nx.connected_components(graph)]
+
+        labels = np.zeros(len(struc))
+        for i, subgraph in enumerate(subgraphs):
+            for n in subgraph:
+                labels[n] = i + 22
+
+        struc.add_site_property("dummy_species", labels.astype(int).tolist())
+        struc.add_site_property(
+            "molecule_index", (labels - 22).astype(int).tolist()
+        )
 
 
 class InterfaceGenerator:
