@@ -65,7 +65,7 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
 
         neighbor_list = []
 
-        cnn = CrystalNN()
+        cnn = CrystalNN(search_cutoff=7.0, cation_anion=True)
         for i, site in enumerate(struc.sites):
             info_dict = cnn.get_nn_info(struc, i)
             for neighbor in info_dict:
@@ -107,6 +107,8 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
 
                 neighbor_dict[n] = d1 + d2
 
+            # print(f"{s1}-{s2} ", neighbor_dict[n])
+
         return neighbor_dict
 
     def _get_r0s(self, sub, film, charge_dict):
@@ -146,7 +148,7 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
 
         for key in unique_keys:
             rev_key = tuple(reversed(key))
-            sum_d = ionic_radius_dict[key[0]] + ionic_radius_dict[key[1]]
+            ionic_sum_d = ionic_radius_dict[key[0]] + ionic_radius_dict[key[1]]
             if key in sub_dict and key in film_dict:
                 sub_d = sub_dict[key]
                 film_d = film_dict[key]
@@ -166,39 +168,41 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
             if key in sub_dict and key not in film_dict:
                 sub_d = sub_dict[key]
                 interface_neighbor_dict[(0, 0) + key] = sub_d
-                interface_neighbor_dict[(1, 1) + key] = sum_d
+                interface_neighbor_dict[(1, 1) + key] = ionic_sum_d
                 interface_neighbor_dict[(0, 1) + key] = sub_d
                 interface_neighbor_dict[(1, 0) + key] = sub_d
                 interface_neighbor_dict[(0, 0) + rev_key] = sub_d
-                interface_neighbor_dict[(1, 1) + rev_key] = sum_d
+                interface_neighbor_dict[(1, 1) + rev_key] = ionic_sum_d
                 interface_neighbor_dict[(0, 1) + rev_key] = sub_d
                 interface_neighbor_dict[(1, 0) + rev_key] = sub_d
 
             if key not in sub_dict and key in film_dict:
                 film_d = film_dict[key]
                 interface_neighbor_dict[(1, 1) + key] = film_d
-                interface_neighbor_dict[(0, 0) + key] = sum_d
+                interface_neighbor_dict[(0, 0) + key] = ionic_sum_d
                 interface_neighbor_dict[(0, 1) + key] = film_d
                 interface_neighbor_dict[(1, 0) + key] = film_d
                 interface_neighbor_dict[(1, 1) + rev_key] = film_d
-                interface_neighbor_dict[(0, 0) + rev_key] = sum_d
+                interface_neighbor_dict[(0, 0) + rev_key] = ionic_sum_d
                 interface_neighbor_dict[(0, 1) + rev_key] = film_d
                 interface_neighbor_dict[(1, 0) + rev_key] = film_d
 
             if key not in sub_dict and key not in film_dict:
-                interface_neighbor_dict[(0, 0) + key] = sub_d
-                interface_neighbor_dict[(1, 1) + key] = sum_d
-                interface_neighbor_dict[(0, 1) + key] = sum_d
-                interface_neighbor_dict[(1, 0) + key] = sum_d
-                interface_neighbor_dict[(0, 0) + rev_key] = sub_d
-                interface_neighbor_dict[(1, 1) + rev_key] = sum_d
-                interface_neighbor_dict[(0, 1) + rev_key] = sum_d
-                interface_neighbor_dict[(1, 0) + rev_key] = sum_d
+                interface_neighbor_dict[(0, 0) + key] = ionic_sum_d
+                interface_neighbor_dict[(1, 1) + key] = ionic_sum_d
+                interface_neighbor_dict[(0, 1) + key] = ionic_sum_d
+                interface_neighbor_dict[(1, 0) + key] = ionic_sum_d
+                interface_neighbor_dict[(0, 0) + rev_key] = ionic_sum_d
+                interface_neighbor_dict[(1, 1) + rev_key] = ionic_sum_d
+                interface_neighbor_dict[(0, 1) + rev_key] = ionic_sum_d
+                interface_neighbor_dict[(1, 0) + rev_key] = ionic_sum_d
 
         for key, val in interface_neighbor_dict.items():
             if val is None:
-                sum_d = ionic_radius_dict[key[2]] + ionic_radius_dict[key[3]]
-                interface_neighbor_dict[key] = sub_d
+                ionic_sum_d = (
+                    ionic_radius_dict[key[2]] + ionic_radius_dict[key[3]]
+                )
+                interface_neighbor_dict[key] = ionic_sum_d
 
         return interface_neighbor_dict
 
@@ -308,18 +312,30 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
         show_max: bool = False,
     ) -> float:
         shifts = self.shifts
-        atoms_list = self._get_shifted_atoms(shifts)
-        inputs = self._generate_inputs(atoms_list)
+        batch_atoms_list = [self._get_shifted_atoms(shift) for shift in shifts]
+        batch_inputs = [self._generate_inputs(b) for b in batch_atoms_list]
+
+        # atoms_list = self._get_shifted_atoms(shifts)
+        # inputs = self._generate_inputs(atoms_list)
 
         x_grid = np.linspace(0, 1, self.grid_density_x)
         y_grid = np.linspace(0, 1, self.grid_density_y)
         X, Y = np.meshgrid(x_grid, y_grid)
 
         if self.z_PES_data is None:
-            z_coulomb_energy = self._calculate_coulomb(inputs, z_shift=True)
-            z_born_energy = self._calculate_born(inputs, z_shift=True)
-            z_interface_coulomb_energy = z_coulomb_energy.reshape(X.shape)
-            z_interface_born_energy = z_born_energy.reshape(X.shape)
+            z_interface_coulomb_energy = np.vstack(
+                [
+                    self._calculate_coulomb(b, z_shift=True)
+                    for b in batch_inputs
+                ]
+            )
+            z_interface_born_energy = np.vstack(
+                [self._calculate_born(b, z_shift=True) for b in batch_inputs]
+            )
+            # z_coulomb_energy = self._calculate_coulomb(inputs, z_shift=True)
+            # z_born_energy = self._calculate_born(inputs, z_shift=True)
+            # z_interface_coulomb_energy = z_coulomb_energy.reshape(X.shape)
+            # z_interface_born_energy = z_born_energy.reshape(X.shape)
             self.z_PES_data = [
                 z_interface_coulomb_energy,
                 z_interface_born_energy,
@@ -328,11 +344,17 @@ class IonicSurfaceMatcher(BaseSurfaceMatcher):
             z_interface_coulomb_energy = self.z_PES_data[0]
             z_interface_born_energy = self.z_PES_data[1]
 
-        coulomb_energy = self._calculate_coulomb(inputs, z_shift=False)
-        born_energy = self._calculate_born(inputs, z_shift=False)
+        interface_coulomb_energy = np.vstack(
+            [self._calculate_coulomb(b, z_shift=False) for b in batch_inputs]
+        )
+        interface_born_energy = np.vstack(
+            [self._calculate_born(b, z_shift=False) for b in batch_inputs]
+        )
 
-        interface_coulomb_energy = coulomb_energy.reshape(X.shape)
-        interface_born_energy = born_energy.reshape(X.shape)
+        # coulomb_energy = self._calculate_coulomb(inputs, z_shift=False)
+        # born_energy = self._calculate_born(inputs, z_shift=False)
+        # interface_coulomb_energy = coulomb_energy.reshape(X.shape)
+        # interface_born_energy = born_energy.reshape(X.shape)
 
         coulomb_adh_energy = (
             z_interface_coulomb_energy - interface_coulomb_energy
